@@ -20,7 +20,8 @@ import (
 	"strings"
 	"testing"
 
-	monitoringv1 "github.com/coreos/prometheus-operator/pkg/apis/monitoring/v1"
+	monitoringv1 "github.com/prometheus-operator/prometheus-operator/pkg/apis/monitoring/v1"
+	"github.com/prometheus-operator/prometheus-operator/pkg/operator"
 	"github.com/stretchr/testify/require"
 	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
@@ -31,13 +32,14 @@ import (
 )
 
 var (
-	defaultTestConfig = &Config{
-		ConfigReloaderImage:           "jimmidyson/configmap-reload:latest",
-		ConfigReloaderCPU:             "100m",
-		ConfigReloaderMemory:          "25Mi",
-		PrometheusConfigReloaderImage: "quay.io/coreos/prometheus-config-reloader:latest",
-		PrometheusDefaultBaseImage:    "quay.io/prometheus/prometheus",
-		ThanosDefaultBaseImage:        "quay.io/thanos/thanos",
+	defaultTestConfig = &operator.Config{
+		ReloaderConfig: operator.ReloaderConfig{
+			CPU:    "100m",
+			Memory: "25Mi",
+			Image:  "quay.io/prometheus-operator/prometheus-config-reloader:latest",
+		},
+		PrometheusDefaultBaseImage: "quay.io/prometheus/prometheus",
+		ThanosDefaultBaseImage:     "quay.io/thanos/thanos",
 	}
 )
 
@@ -67,12 +69,12 @@ func TestStatefulSetLabelingAndAnnotations(t *testing.T) {
 	require.NoError(t, err)
 
 	if !reflect.DeepEqual(labels, sset.Labels) {
-		fmt.Println(pretty.Compare(labels, sset.Labels))
+		t.Log(pretty.Compare(labels, sset.Labels))
 		t.Fatal("Labels are not properly being propagated to the StatefulSet")
 	}
 
 	if !reflect.DeepEqual(expectedAnnotations, sset.Annotations) {
-		fmt.Println(pretty.Compare(expectedAnnotations, sset.Annotations))
+		t.Log(pretty.Compare(expectedAnnotations, sset.Annotations))
 		t.Fatal("Annotations are not properly being propagated to the StatefulSet")
 	}
 }
@@ -101,6 +103,25 @@ func TestPodLabelsAnnotations(t *testing.T) {
 		t.Fatal("Pod annotaitons are not properly propagated")
 	}
 }
+func TestPodLabelsShouldNotBeSelectorLabels(t *testing.T) {
+	labels := map[string]string{
+		"testlabel": "testvalue",
+	}
+	sset, err := makeStatefulSet(monitoringv1.Prometheus{
+		ObjectMeta: metav1.ObjectMeta{},
+		Spec: monitoringv1.PrometheusSpec{
+			PodMetadata: &monitoringv1.EmbeddedObjectMetadata{
+				Labels: labels,
+			},
+		},
+	}, defaultTestConfig, nil, "")
+
+	require.NoError(t, err)
+
+	if sset.Spec.Selector.MatchLabels["testlabel"] == "testvalue" {
+		t.Fatal("Pod Selector are not properly propagated")
+	}
+}
 
 func TestStatefulSetPVC(t *testing.T) {
 	labels := map[string]string{
@@ -479,6 +500,26 @@ func TestTagAndShaAndVersion(t *testing.T) {
 			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, image)
 		}
 	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				SHA:     "7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324",
+				Tag:     "my-unrelated-tag",
+				Version: "v2.3.2",
+				Image:   &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "my-reg/prometheus@sha256:7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
 	{
 		image := "my-reg/prometheus:latest"
 		sset, err := makeStatefulSet(monitoringv1.Prometheus{
@@ -499,6 +540,115 @@ func TestTagAndShaAndVersion(t *testing.T) {
 			t.Fatalf("Explicit image should have precedence. Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
 		}
 	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				Version: "v2.3.2",
+				Image:   &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "my-reg/prometheus:v2.3.2"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				SHA:     "7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324",
+				Version: "v2.3.2",
+				Image:   &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "my-reg/prometheus@sha256:7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				Image: &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "my-reg/prometheus"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				SHA:   "7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324",
+				Image: &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "my-reg/prometheus@sha256:7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				Tag:   "my-unrelated-tag",
+				Image: &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "docker.io/my-reg/prometheus:my-unrelated-tag"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
+	{
+		image := "my-reg/prometheus"
+		sset, err := makeStatefulSet(monitoringv1.Prometheus{
+			Spec: monitoringv1.PrometheusSpec{
+				SHA:   "7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324",
+				Tag:   "my-unrealted-tag",
+				Image: &image,
+			},
+		}, defaultTestConfig, nil, "")
+		if err != nil {
+			t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+		}
+
+		resultImage := sset.Spec.Template.Spec.Containers[0].Image
+		expected := "my-reg/prometheus@sha256:7384a79f4b4991bf8269e7452390249b7c70bcdd10509c8c1c6c6e30e32fb324"
+		if resultImage != expected {
+			t.Fatalf("Unexpected container image.\n\nExpected: %s\n\nGot: %s", expected, resultImage)
+		}
+	}
 }
 
 func TestThanosTagAndShaAndVersion(t *testing.T) {
@@ -707,6 +857,58 @@ func TestThanosObjectStorage(t *testing.T) {
 	}
 }
 
+func TestThanosTracing(t *testing.T) {
+	testKey := "thanos-config-secret-test"
+
+	sset, err := makeStatefulSet(monitoringv1.Prometheus{
+		Spec: monitoringv1.PrometheusSpec{
+			Thanos: &monitoringv1.ThanosSpec{
+				TracingConfig: &v1.SecretKeySelector{
+					Key: testKey,
+				},
+			},
+		},
+	}, defaultTestConfig, nil, "")
+	if err != nil {
+		t.Fatalf("Unexpected error while making StatefulSet: %v", err)
+	}
+
+	if sset.Spec.Template.Spec.Containers[0].Name != "prometheus" {
+		t.Fatalf("expected 1st containers to be prometheus, got %s", sset.Spec.Template.Spec.Containers[0].Name)
+	}
+
+	if sset.Spec.Template.Spec.Containers[2].Name != "thanos-sidecar" {
+		t.Fatalf("expected 3rd containers to be thanos-sidecar, got %s", sset.Spec.Template.Spec.Containers[2].Name)
+	}
+
+	var containsEnvVar bool
+	for _, env := range sset.Spec.Template.Spec.Containers[2].Env {
+		if env.Name == "TRACING_CONFIG" {
+			if env.ValueFrom.SecretKeyRef.Key == testKey {
+				containsEnvVar = true
+				break
+			}
+		}
+	}
+	if !containsEnvVar {
+		t.Fatalf("Thanos sidecar is missing expected TRACING_CONFIG env var with correct value")
+	}
+
+	{
+		var containsArg bool
+		const expectedArg = "--tracing.config=$(TRACING_CONFIG)"
+		for _, arg := range sset.Spec.Template.Spec.Containers[2].Args {
+			if arg == expectedArg {
+				containsArg = true
+				break
+			}
+		}
+		if !containsArg {
+			t.Fatalf("Thanos sidecar is missing expected argument: %s", expectedArg)
+		}
+	}
+}
+
 func TestRetentionSize(t *testing.T) {
 	tests := []struct {
 		version              string
@@ -793,13 +995,14 @@ func TestRetention(t *testing.T) {
 }
 
 func TestSidecarsNoCPULimits(t *testing.T) {
-	testConfig := &Config{
-		ConfigReloaderImage:           "jimmidyson/configmap-reload:latest",
-		ConfigReloaderCPU:             "0",
-		ConfigReloaderMemory:          "50Mi",
-		PrometheusConfigReloaderImage: "quay.io/coreos/prometheus-config-reloader:latest",
-		PrometheusDefaultBaseImage:    "quay.io/prometheus/prometheus",
-		ThanosDefaultBaseImage:        "quay.io/thanos/thanos:v0.7.0",
+	testConfig := &operator.Config{
+		ReloaderConfig: operator.ReloaderConfig{
+			CPU:    "0",
+			Memory: "50Mi",
+			Image:  "quay.io/prometheus-operator/prometheus-config-reloader:latest",
+		},
+		PrometheusDefaultBaseImage: "quay.io/prometheus/prometheus",
+		ThanosDefaultBaseImage:     "quay.io/thanos/thanos:v0.7.0",
 	}
 	sset, err := makeStatefulSet(monitoringv1.Prometheus{
 		Spec: monitoringv1.PrometheusSpec{},
@@ -808,24 +1011,30 @@ func TestSidecarsNoCPULimits(t *testing.T) {
 		t.Fatalf("Unexpected error while making StatefulSet: %v", err)
 	}
 
-	expectedResources := v1.ResourceRequirements{Limits: v1.ResourceList{
-		v1.ResourceMemory: resource.MustParse("50Mi"),
-	}}
+	expectedResources := v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceMemory: resource.MustParse("50Mi"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceMemory: resource.MustParse("50Mi"),
+		},
+	}
 	for _, c := range sset.Spec.Template.Spec.Containers {
 		if (c.Name == "prometheus-config-reloader" || c.Name == "rules-configmap-reloader") && !reflect.DeepEqual(c.Resources, expectedResources) {
-			t.Fatal("Unexpected resource requests/limits set, when none should be set.")
+			t.Fatalf("Expected resource requests/limits:\n\n%s\n\nGot:\n\n%s", expectedResources.String(), c.Resources.String())
 		}
 	}
 }
 
 func TestSidecarsNoMemoryLimits(t *testing.T) {
-	testConfig := &Config{
-		ConfigReloaderImage:           "jimmidyson/configmap-reload:latest",
-		ConfigReloaderCPU:             "100m",
-		ConfigReloaderMemory:          "0",
-		PrometheusConfigReloaderImage: "quay.io/coreos/prometheus-config-reloader:latest",
-		PrometheusDefaultBaseImage:    "quay.io/prometheus/prometheus",
-		ThanosDefaultBaseImage:        "quay.io/thanos/thanos:v0.7.0",
+	testConfig := &operator.Config{
+		ReloaderConfig: operator.ReloaderConfig{
+			CPU:    "100m",
+			Memory: "0",
+			Image:  "quay.io/prometheus-operator/prometheus-config-reloader:latest",
+		},
+		PrometheusDefaultBaseImage: "quay.io/prometheus/prometheus",
+		ThanosDefaultBaseImage:     "quay.io/thanos/thanos:v0.7.0",
 	}
 	sset, err := makeStatefulSet(monitoringv1.Prometheus{
 		Spec: monitoringv1.PrometheusSpec{},
@@ -834,12 +1043,17 @@ func TestSidecarsNoMemoryLimits(t *testing.T) {
 		t.Fatalf("Unexpected error while making StatefulSet: %v", err)
 	}
 
-	expectedResources := v1.ResourceRequirements{Limits: v1.ResourceList{
-		v1.ResourceCPU: resource.MustParse("100m"),
-	}}
+	expectedResources := v1.ResourceRequirements{
+		Limits: v1.ResourceList{
+			v1.ResourceCPU: resource.MustParse("100m"),
+		},
+		Requests: v1.ResourceList{
+			v1.ResourceCPU: resource.MustParse("100m"),
+		},
+	}
 	for _, c := range sset.Spec.Template.Spec.Containers {
 		if (c.Name == "prometheus-config-reloader" || c.Name == "rules-configmap-reloader") && !reflect.DeepEqual(c.Resources, expectedResources) {
-			t.Fatal("Unexpected resource requests/limits set, when none should be set.")
+			t.Fatalf("Expected resource requests/limits:\n\n%s\n\nGot:\n\n%s", expectedResources.String(), c.Resources.String())
 		}
 	}
 }
@@ -847,6 +1061,7 @@ func TestSidecarsNoMemoryLimits(t *testing.T) {
 func TestAdditionalContainers(t *testing.T) {
 	// The base to compare everything against
 	baseSet, err := makeStatefulSet(monitoringv1.Prometheus{}, defaultTestConfig, nil, "")
+	require.NoError(t, err)
 
 	// Add an extra container
 	addSset, err := makeStatefulSet(monitoringv1.Prometheus{
